// Ansteuern eines Standard-LCD mit 4-Bit-Schnittstelle
// Pin 1.0: E
// Pin 1.1: RS
// Pins 1.4 bis 1.7: D4 bis D7
// 2010-11-19: von jl erstellt

#include "display01.h"
#include "io430.h"
#include <stdbool.h>

static const unsigned char RS = BIT0;
static const unsigned char E = BIT1;

// Daten in den oberen vier Bits
static void sendNibble(unsigned char data)
{
  P1OUT = data;
  P1OUT |= E;
  waitOneMillisecond();
  P1OUT &= ~E;  
  waitOneMillisecond(); // schneller wre, auf die Rckmeldung des Displays zu warten,
                        // aber so sparen wir eine Leitung
}

static void sendByte(unsigned char data)
{
  sendNibble(data & 0xF0u);
  sendNibble(data << 4);
}

static void sendByteRAM(unsigned char data)
{
  sendNibble(data & 0xF0u | RS);
  sendNibble((data << 4) | RS);
}

void initialize(void)
{
  WDTCTL = WDTPW | WDTHOLD;
  
  BCSCTL1 = CALBC1_1MHZ;
  DCOCTL = CALDCO_1MHZ;

  TACTL = MC_2 | TASSEL_2 | TACLR;

  P1OUT = 0;  
  P1DIR |= 0xF7;
  waitMilliseconds(50);
  sendNibble(0x30u); // Initialisierungsequenz fr den Warmstart
  waitMilliseconds(50);
  sendNibble(0x30u);
  waitMilliseconds(50);
  sendNibble(0x30u);
  sendNibble(0x20u);
  sendByte(0x20u); // Ansteuerung nur einer Zeile, weil fr unser LCD sonst negative Kontrastspannung ntig
  sendByte(0x01u); // lschen  
  sendByte(0x0Cu); // Display an, Cursor aus, Blinken aus
  sendByte(0x06u); // den Inhalt nicht mit dem Cursor schieben
}

void clear(void)
{
  sendByte(0x01u);
}

void putCharacter(unsigned char row, unsigned char column, unsigned char character)
{
  sendByte(0x02u); // "home"
  sendByte(0x80u + 0x40u*row + column); // Adresse
  sendByteRAM(character);
}

void showCursor(bool visible, bool blinking)
{
  sendByte(0x0Cu | (visible?2u:0u) | (blinking?1u:0u));
}

void shiftCursor(bool right)
{
  sendByte(0x10u | (right?0u:1u));
}

void moveCursor(unsigned char row, unsigned char column)
{
  sendByte(0x02u); // "return home"  
  sendByte(0x80u + column + 0x40u*row);
}

void waitOneMillisecond(void)
{
  unsigned short tStart = TAR;
  unsigned short tEnd = tStart + 1000;
  // Workaround, weil TAR nur bis 0xFFFE zhlt
  if(tEnd == 0xFFFF)
  {
    tEnd--;
  }
  
  if(tEnd < tStart) // Hatten wir einen berlauf beim Berechnen von tEnd?
  {
    while(TAR > tEnd)
    {}
  }
  while(TAR < tEnd)
  {}    
}

void waitMilliseconds(unsigned int duration)
{
  for(unsigned int i = 0; i < duration; i++)
  {
    waitOneMillisecond();
  }
}